#!/bin/sh
#
# This is a modified version of the original script for starting and 
# stoping squeezelite created by pCP:
#   /usr/local/etc/init.d/squeezelite
#
# This script will start squeezelite with its output forced to "stdout",
# ignoring the output settings in pCP's web-interface. The output on
# stdout is then piped to CamillaDSP stdin. 
# The squeezelite log level and file is also forced to: "output=info",
# to a "named pipe" file, ignoring the settings in pCP's web-interface.
# The log output from squeezelite is sent through the named pipe, to a
# python daemon, who catches messages of new sample rate values, and
# sends a command to CamillaDSP to reload a new config file for the
# new sample rate.
#

if [[ "$USER" != 'root' && "$1" != "status" ]] ; then
    echo
    echo "This script must be run as root, or with sudo."
    echo "Usage:   sudo `basename $0` start|stop|restart|status"
    echo
    exit
fi

cdspDIR=/opt/camilladsp
cdspPORT="1234"
cdspCFGDIR=/home/tc/camilladsp
cdspCFG="44100.yml"
cdspLOGFILE=/var/log/camilladsp.log
pyDAEMON="$cdspDIR/samplerate-daemon.py"
pyPATH=/usr/local/bin/python3
pipeFILE=/var/run/squeezelite-log.fifo
LOGLEVELpipe=" output=info"
OUTPUTcdsp=" -o -"

TCEMNT="/mnt/$(readlink /etc/sysconfig/tcedir | cut -d '/' -f3)"
PNAME=Squeezelite
DESC="Squeezelite player and CamillaDSP"
# Set DAEMON to the actual binary of squeezelite. Will NOT be run as a daemon!
DAEMON=/usr/local/bin/squeezelite
PIDFILE=/var/run/squeezelite.pid
sqLOGFILE=/var/log/pcp_squeezelite.log

start () {
    echo "+ Starting $DESC..."
    
    # Read from config file
    . /usr/local/etc/pcp/pcp.cfg

    # Check if variable is present then add the correct option in front
    [ x"" != x"$NAME" ]         && NAME="$NAME"							# <--- Moved "-n" to allow for spaces
    [ x"" != x"$OUTPUT" ]       && OUTPUT=" -o $OUTPUT"
    [ x"" != x"$ALSA_PARAMS" ]  && ALSA_PARAMS=" -a "$ALSA_PARAMS""
    [ x"" != x"$BUFFER_SIZE" ]  && BUFFER_SIZE=" -b $BUFFER_SIZE"
    [ x"" != x"$_CODEC" ]       && _CODEC=" -c $_CODEC"
    [ x"" != x"$XCODEC" ]       && XCODEC=" -e $XCODEC"
    [ x"" != x"$PRIORITY" ]     && PRIORITY=" -p $PRIORITY"
    [ x"" != x"$MAX_RATE" ]     && MAX_RATE=" -r $MAX_RATE"
    [ x"" != x"$UPSAMPLE" ]     && UPSAMPLE=" -R $UPSAMPLE"
    [ x"" != x"$MAC_ADDRESS" ]  && MAC_ADDRESS=" -m $MAC_ADDRESS"
    [ x"" != x"$SERVER_IP" ]    && SERVER_IP=" -s $SERVER_IP"
    #[ x"" != x"$LOGLEVEL" ]     && LOGLEVEL="-d $LOGLEVEL -f ${sqLOGFILE}"
    #[ x"" != x"$LOGLEVEL" ]     && LOGLEVEL=" -d ${LOGLEVELpipe} -f ${pipeFILE}"
    [ x"" != x"$LOGLEVEL" ]     && LOGLEVEL=" -d ${LOGLEVEL}"
    [ x"" != x"$DSDOUT" ]       && DSDOUT=" -D $DSDOUT"
    [ "$VISUALISER" = "yes" ]     && VIS=" -v"
    [ x"" != x"$CLOSEOUT" ]     && CLOSEOUT=" -C $CLOSEOUT"
    [ x"" != x"$UNMUTE" ]       && UNMUTE=" -U $UNMUTE"
    [ x"" != x"$ALSAVOLUME" ]   && ALSAVOLUME=" -V $ALSAVOLUME"
    [ "$IR_LIRC" = "yes" ] && [ "$JIVELITE" != "yes" ] && LIRC=" -i $IR_CONFIG"
    [ x"" != x"$POWER_GPIO" ]   && POWER_GPIO=" -G $POWER_GPIO:$POWER_OUTPUT"
    [ x"" != x"$POWER_SCRIPT" ] && POWER_SCRIPT=" -S $POWER_SCRIPT"
    [ x"" != x"$OTHER" ]        && OTHER=" $OTHER"
    OUTPUT=$OUTPUTcdsp

    [ $1 ] && cLOG=$1
    cLOG=${cLOG//-/}
    [ ${#cLOG} -gt 2 ] && usage "  Error - invalid option." && exit 1
    case "$cLOG" in
	l)
		cdspLOGLEVEL=""
        ;;
	v)
        cdspLOGLEVEL="-v"
        ;;
	lv|vl)
        cdspLOGLEVEL="-v"
        ;;
    *)
        [ -n $cLOG ] && usage "  Error - invalid option." && exit 1
        cdspLOGFILE=/dev/null
        ;;
    esac
    OTHER2=" | $cdspDIR/camilladsp $cdspLOGLEVEL -p $cdspPORT $cdspCFGDIR/$cdspCFG > $cdspLOGFILE 2>&1"
    # echo "OTHER2:  $OTHER2"

    # Check if squeezelite is running, started by pCP
    if [ -f $PIDFILE ]; then
        echo "$PNAME is already started by pCP."
        exit 1
    fi
    # Check if squeezelite is running, stared elsewhere
    if [ -n "`ps | grep -v grep | grep $DAEMON`" ] ; then
        echo "squeezelite is already running."
        exit 1
    fi
    
    # Check if named pipe for pyDAEMON needs to be created
    if [ ! -p "$pipeFILE" ] ; then
        # Check if normal file exist, and remove it
        if [ -f "$pipeFILE" ] ; then
            echo "Removing normal file $pipeFILE (blocking creation of named pipe $pipeFILE)"
            rm "$pipeFILE"
        fi
        mkfifo "$pipeFILE"
    fi
    
    # Start pyDAEMON. Restart if running
    x=`ps | grep -v grep | grep $pyDAEMON`     # empty bash/ps cache if pyDaemon recently is killed
    if [ -n "`ps | grep -v grep | grep $pyDAEMON`" ] ; then
        kill $(pgrep -f $pyDAEMON)
    fi
    if [ "$LOGLEVEL" = "" ] ; then
        # If loglevel is "none", disable output to sqLOGFILE
        sqLOGFILE=""
        # Set minimum loglevel to get sample rate info to pyDAEMON
        LOGLEVEL=" -d output=info"
    elif [ "${LOGLEVEL%%=*}" != " -d output" ] ; then
        # All log categories but "output" must be replaced to "all". (output must allways be included)
        LOGLEVEL=" -d all=${LOGLEVEL##*=}"
    fi
    echo "+ Starting daemon $pyDAEMON"
    eval "$pyPATH $pyDAEMON $sqLOGFILE &"
    
    # Start squeezelite
    pipeFILE=" -f $pipeFILE"
    cmd="$DAEMON -n "$NAME"$OUTPUT$ALSA_PARAMS$BUFFER_SIZE$_CODEC$XCODEC$PRIORITY$MAX_RATE$UPSAMPLE$MAC_ADDRESS$SERVER_IP$DSDOUT$VIS$CLOSEOUT$UNMUTE$ALSAVOLUME$LIRC$POWER_GPIO$POWER_SCRIPT$OTHER$LOGLEVEL$pipeFILE$OTHER2 &"
    # echo "cmd= $cmd"
    eval "$cmd"
    
    # Check for running processes
    c=`pgrep -f $cdspDIR/camilladsp`
    s=`pgrep -f $DAEMON`
    d=`pgrep -f $pyDAEMON`
    [ ! $c ] && echo "  Error - Could not start $cdspDIR/camilladsp."
    [ ! $s ] && echo "  Error - Could not start $DAEMON."
    [ ! $d ] && echo "  Error - Could not start $pyDAEMON."
    #echo "return: $((c+s+d))"
    return $((c+s+d))
}

stop () {
    echo "- Stopping $DESC..."
    #if [ -z "`ps | grep -v grep | grep $DAEMON`" ] ; then
    #    echo "$DESC is not running."
    #    exit 1
    #fi
    #killall camilladsp
    
    c=`pgrep -f $cdspDIR/camilladsp`
    [ $c ] && kill $c && sleep 1
    # If killing camilladsp didn't stop all processes, try kill the rest.
    s=`pgrep -f $DAEMON`
    [ $s ] && kill $s && sleep 1
    d=`pgrep -f $pyDAEMON`
    [ $d ] && kill $d && sleep 1
    # Check for still running processes
    c=`pgrep -f $cdspDIR/camilladsp`
    s=`pgrep -f $DAEMON`
    d=`pgrep -f $pyDAEMON`
    [ $c ] && echo "  Error - Could not stop $cdspDIR/camilladsp."
    [ $s ] && echo "  Error - Could not stop $DAEMON."
    [ $d ] && echo "  Error - Could not stop $pyDAEMON."
    #echo "return: $((c+s+d))"
    return $((c+s+d))
}

usage () {
		[ "$1" ] && echo "$1"
        echo
		echo "Usage: sudo `basename $0` start|stop|restart|status [-l] [-v]"
		echo 
        echo "        -l      Activate camilladsp log to $cdspLOGFILE"
        echo "        -v      Verbose log"
        echo
}

case "$1" in
	start)
        o=""
        [ $2 ] && o=$2
        [ $3 ] && o="$o$3"
        start $o
        ;;
	stop)
		stop
        ;;
	restart)
		o=""
        [ $2 ] && o=$2
        [ $3 ] && o="$o$3"
        echo "Restarting $DESC..."
		stop
        #echo "?  $?"
        [ $? -gt 0 ] && echo -e "\nRestarting canceled.\n" && exit 1
		start $o
		;;
	status)
		# Check if our squeezelite daemon is running.
        me=`basename $0`
        ps="`ps -o pid,ppid,user,time,args | grep -v grep | grep -v $me | grep -i 'camilladsp\|squeezelite'`"
		if [ -z "$ps" ]; then
            echo
            echo "$DESC is not running."
            echo
            exit
		fi
		echo
        echo "Running processes:"
        echo -e "PID:  PPID: User:  cpuTime: Command:"
        echo "$ps"
		echo
		;;
	*)
		usage
        ;;
esac

exit



# Example method for using pid-files...
#if [[ ! -e /tmp/test.py.pid ]]; then   # Check if the file already exists
    #python test.py &                   #+and if so do not run another process.
    #echo $! > /tmp/test.py.pid
#else
    #echo -n "ERROR: The process is already running with pid "
    #cat /tmp/test.py.pid
    #echo
#fi
