#!/bin/sh

# This script will switch and apply configuration for CamillaDSP.
# ---------------------------------------------------------------
# Configurations, with a set of files for different samplerates,
# must be saved in individual subfolders to: "ConfigPATH"
#   (default: /home/tc/DSP_Engine/filters).
# Example:
#  - /home/tc/DSP_Engine/filters
#     - config_1
#        - 44100.yml
#        - 96000.yml
#        - ...
#     - config_2
#        - 44100.yml
#        - 96000.yml
#        - ...
# Symbolic links to each samplerate file in the selected subfolder is
# first created or overwritten in the base folder "ConfigPATH". 
# SIGHUP is then sent to CamillaDSP that will reload it's config and
# apply the new settings.
#   (CamillaDSP must allways read it's config from the sybolic link files
#    in the base folder "ConfigPATH")
# ---------------------------------------------------------------


# Show comment from first n lines in file *44100.yml. Set to 0 to disable
ShowComment=4
ConfigPATH="/home/tc/camilladsp"

if [ ! $1 ] ; then
  Me=`basename $0`
  echo -e "-----\nError - Name of folder with configuration files is missing.\n-----"
  echo "Usage exemple:"
  echo -e "  $Me config_1\n"
  echo "$0"
  exit 1
fi

set -e    # set exit-on-error mode.
ConfigFolder=`basename $1`

if [ ! -d "$ConfigPATH/$ConfigFolder" ] ; then
  echo -e "-----\nError - The folder: '$ConfigPATH/$ConfigFolder' don't exist.\n-----"
  echo "The configuration folder must be located in: $ConfigPATH"
  echo "List of existing folders:"
  for d in $ConfigPATH/* ; do [ -d "$d" ] && echo "- ${d##*[/]}" ; done
  exit 1
fi

for f in $ConfigPATH/$ConfigFolder/*.yml ; do 
  if [ -f $f ] ; then
    if [ ! $Label ] ; then
      echo -e "\nSetting CamillaDSP configuration to: $ConfigFolder"
      if [ $ShowComment > 0 ] ; then
        echo "config comment:"
        echo "$(head -$ShowComment $ConfigPATH/$ConfigFolder/*44100.yml | sed -e 's/^/  /')"
      fi
      echo -e "\nCreates or updates symbolic links in folder: $ConfigPATH"
      Label=1
    fi
    File=`basename $f`
    echo "  $File -> $ConfigFolder/$File"
    ln -sf $ConfigFolder/$File $ConfigPATH/$File
  else
    echo -e "-----\nError - There are no config files (*.yml) in folder: '$ConfigPATH/$ConfigFolder'"
    exit 1
  fi
done

echo -e "\nSending signal to CamillaDSP to apply configuration: $ConfigFolder\n"
sudo killall -s HUP camilladsp

